local LSM = LibStub("LibSharedMedia-3.0")

---@type SchtenksBarsConsts
local Const = SchtenksBars.Const

Const.UI = {}
Const.UI.BorderTexture = "Interface\\BUTTONS\\WHITE8X8"
Const.UI.SparkTexture = "Interface\\CastingBar\\UI-CastingBar-Spark"
Const.UI.DefaultIcon = "Interface\\TutorialFrame\\UI-TutorialFrame-Hearthstone"

SchtenksBars.UI = {}

-- hostile = {r = 0.90, g = 0.0, b = 0.0},
-- aggro = {r = 0.90, g = 0.0, b = 0.0},
-- friendly = {r = 0.20, g = 0.90, b = 0.20},
-- neutral = {r = 0.93, g = 0.93, b = 0.0},

function SchtenksBars.UI:GetBarColors(settings, BarName, class, canUseHostility)
	if settings == nil then
		settings = SchtenksBars.DB.profile[BarName]
		if settings[Const.DB.UseGeneral] then
		 	settings = SchtenksBars.DB.profile[Const.General]
		end
	end
	local FGColor = {}
	local BGColor = {}
	if settings[Const.DB.EnableClass] and class ~= nil then
		local R, G, B, _ = GetClassColor(class)
		FGColor.R = R
		FGColor.G = G
		FGColor.B = B
		FGColor.A = 1

		BGColor.R = R * 0.4
		BGColor.G = G * 0.4
		BGColor.B = B * 0.4
		BGColor.A = 1
	elseif settings[Const.DB.HostilityBased] and canUseHostility then
		local r,g,b,a = UnitSelectionColor(BarName, true)
		FGColor.R = r
		FGColor.G = g
		FGColor.B = b
		FGColor.A = a

		BGColor.R = r * 0.4
		BGColor.G = g * 0.4
		BGColor.B = b * 0.4
		BGColor.A = a
	else
		FGColor = settings[Const.DB.ForegroundColor]
		BGColor = settings[Const.DB.BackgroundColor]
	end
	return FGColor, BGColor
end

function SchtenksBars.UI:GetBorderTemplate(BorderThickness)
	---@type SchtenksBarsConsts
	local Const = SchtenksBars.Const
	local template = {
		--bgFile = "Interface\\BUTTONS\\WHITE8X8",
		edgeFile = Const.UI.BorderTexture,
		--tile = false,
		tileEdge = false,
		--tileSize = 0,
		edgeSize = BorderThickness,
		insets = {left = -BorderThickness, top = -BorderThickness, right = -BorderThickness, bottom = -BorderThickness}
	}
	return template
end

---@param self Bar
function SchtenksBars.UI:UpdateFromSettings(self, settings, SetDefaults)
	if settings == nil then
		settings = {}
		if SchtenksBars.DB.profile[self.BarName][Const.DB.UseGeneral] then
			settings = SchtenksBars:GetSpecificsForBar(settings, self.BarName)
			setmetatable(settings, {__index = SchtenksBars.DB.profile[Const.General]})
		else
			settings = SchtenksBars.DB.profile[self.BarName]
		end
	end

	self.Enabled = settings[Const.DB.Enabled]
	if not self.Enabled then
		self:Hide()
		return
	end
	self:SetClampedToScreen(true)
	self.IconEnabled = settings[Const.DB.ShowIcon]
	self:ShowIcon(self.IconEnabled)

	self:SetSize(settings[Const.DB.Width], settings[Const.DB.Heigth], settings[Const.DB.BorderThickness], self.IconEnabled)
	self:SetPosition(settings[Const.DB.AnchorPoint], settings[Const.DB.AnchorFrame], settings[Const.DB.AnchorToPoint], settings[Const.DB.X], settings[Const.DB.Y])
	self:SetSubPositions(settings[Const.DB.BorderThickness], settings[Const.DB.IconOnTheRight])

	self.SparkEnabled = settings[Const.DB.ShowSpark]
	self:ShowSpark(self.SparkEnabled)

	self:SetReverseFill(settings[Const.DB.ReverseFill])
	self:SetForegroundTexture(settings[Const.DB.BarTexture])
	self:SetBackgroundTexture(settings[Const.DB.BarTexture])

	local class, _  = UnitClassBase(Const.Player)
	local FGColor, BGColor = SchtenksBars.UI:GetBarColors(settings, self.BarName, class, self.CanUseHostility)

	self:SetForegroundColor(FGColor)
	self:SetBackgroundColor(BGColor)

	self:SetIconZoom(settings[Const.DB.IconZoom])
	self:SetFont(settings[Const.DB.Font], settings[Const.DB.FontSize])

	self:SetBorderSize(settings[Const.DB.BorderThickness])
	self:SetBorderColor(settings[Const.DB.BorderColor])

	--Set some default values for configuration
	if SetDefaults then
		self:SetIcon(Const.UI.DefaultIcon)
		self:SetNameText("Hearthstone")
		self:SetFullDuration(10)
		self:SetDuration(5)
	end
end

---@return Bar
function SchtenksBars.UI:CreateBarBase()
	---@class Bar
	local Bar = {
		Type = "Bar",
		Enabled = false,
		IconEnabled = false,
		SparkEnabled = false,
		CanUseHostility = false,
		BarName = "",
		BarId = "",
		---@type Frame
		Frame = nil,
		---@type StatusBar
		BarFrame = nil,
		---@type BackdropTemplateMixin
		Border = nil,
		---@type Frame
		Icon = nil,
		---@type Texture
		IconTexture = nil,
		---@type BackdropTemplateMixin
		IconBorder = nil,
		---@type Texture
		Background = nil,
		---@type Texture
		Spark = nil,
		---@type FontString
		NameText = nil,
		---@type FontString
		DurationText = nil,
		---@param self Bar
		["SetNameText"] = function(self, Name)
			self.NameText:SetText(Name)
		end,
		---@param self Bar
		["SetDurationText"] = function(self, Duration)
			self.DurationText:SetText(string.format("%.1f", Duration))
		end,
		---@param self Bar
		["SetDuration"] = function(self, Duration)
			local min , max = self.BarFrame:GetMinMaxValues()
			if Duration > max then Duration = max end
			if Duration < min then Duration = min end
			Duration = ((self.IsChannel and self.ReverseChannel) and max - Duration or Duration)
			self:SetDurationText(Duration)
			self.BarFrame:SetValue(Duration)
			if self.BarFrame:GetReverseFill() then
				self.Spark:SetPoint("RIGHT", self.BarFrame, "RIGHT", -(self.BarFrame:GetWidth() * (Duration/max) - (self.Spark:GetWidth()/2)), 0)
			else
				self.Spark:SetPoint("LEFT", self.BarFrame, "LEFT", (self.BarFrame:GetWidth() * (Duration/max) - (self.Spark:GetWidth()/2)), 0)
			end
		end,
		---@param self Bar
		["SetFullDuration"] = function(self, Duration)
			self.BarFrame:SetMinMaxValues(0, Duration)
		end,
		---@param self Bar
		["SetFont"] = function(self, Font, FontSize)
			self.NameText:SetFont(LSM:Fetch("font", Font), FontSize)
			self.DurationText:SetFont(LSM:Fetch("font", Font), FontSize)
			self.NameText:SetShadowOffset(1, -1)
			self.DurationText:SetShadowOffset(1, -1)
		end,
		---@param self Bar
		["SetIcon"] = function(self, Texture)
			self.IconTexture:SetTexture(Texture)
		end,
		---@param self Bar
		["ShowIcon"] = function(self, Show)
			if Show then self.Icon:Show() else self.Icon:Hide() end
		end,
		---@param self Bar
		["ShowSpark"] = function(self, Show)
			if Show then self.Spark:Show() else self.Spark:Hide() end
		end,
		---@param self Bar
		["ShowDurationText"] = function(self, Show)
			if Show then self.DurationText:Show() else self.DurationText:Hide() end
		end,
		---@param self Bar
		["SetReverseFill"]= function(self, reverse)
			self.BarFrame:SetReverseFill(reverse)
			self.Spark:ClearAllPoints()
		end,
		---@param self Bar
		["SetSubPositions"] = function(self, BorderThickness, IconOnTheRight)
			self.Icon:ClearAllPoints()
			self.BarFrame:ClearAllPoints()
			if IconOnTheRight then
				self.Icon:SetPoint("TOPRIGHT", self.Frame, "TOPRIGHT", -BorderThickness, -BorderThickness)
				self.BarFrame:SetPoint("BOTTOMLEFT", self.Frame, "BOTTOMLEFT", BorderThickness, BorderThickness)
			else
				self.Icon:SetPoint("BOTTOMLEFT", self.Frame, "BOTTOMLEFT", BorderThickness, BorderThickness)
				self.BarFrame:SetPoint("TOPRIGHT", self.Frame, "TOPRIGHT", -BorderThickness, -BorderThickness)
			end

			self.NameText:SetPoint("LEFT", self.BarFrame, "LEFT", 2, 0)
			self.DurationText:SetPoint("RIGHT", self.BarFrame, "RIGHT", -4, 0)
		end,
		---@param self Bar
		["SetPosition"]= function(self, AnchorPoint, AnchorFrame, AnchorToPoint, X, Y)
			self.Frame:ClearAllPoints()
			if (AnchorFrame == "") or (AnchorFrame == nil) then
				AnchorFrame = "UIParent"
			end
			self.Frame:SetPoint(AnchorPoint, AnchorFrame, AnchorToPoint, X, Y)
			self.Frame:SetParent(AnchorFrame)
		end,
		---@param self Bar
		["SetClampedToScreen"] = function(self, clamped)
			self.Frame:SetClampedToScreen(clamped)
		end,
		---@param self Bar
		["SetIconZoom"] = function(self, zoom)
			zoom = zoom / 2
			self.IconTexture:SetTexCoord(zoom, 1 - zoom, zoom, 1 - zoom)
		end,
		---@param self Bar
		["SetBackgroundTexture"] = function(self,Texture)
			self.Background:SetTexture(LSM:Fetch("statusbar", Texture), "BACKGROUND")
		end,
		---@param self Bar
		["SetForegroundTexture"] = function(self,Texture)
			self.BarFrame:SetStatusBarTexture(LSM:Fetch("statusbar", Texture), "BACKGROUND", 1)
		end,
		---@param self Bar
		["SetForegroundColor"] = function(self, Color)
			self.BarFrame:SetStatusBarColor(Color.R, Color.G, Color.B, Color.A)
		end,
		---@param self Bar
		["SetBackgroundColor"] = function(self, Color)
			self.Background:SetVertexColor(Color.R, Color.G, Color.B, Color.A)
		end,
		---@param self Bar
		["Show"] = function(self)
			self.Frame:Show()
		end,
		---@param self Bar
		["Hide"] = function(self)
			self.Frame:Hide()
		end,
		["GetLeft"] = function(self)
			return self.Frame:GetLeft()
		end,
		["GetTop"] = function(self)
			return self.Frame:GetTop()
		end,
		["GetHeight"] = function(self)
			return self.Frame:GetHeight()
		end,
		---@param self Bar
		["Lock"] = function(self)
			self.Frame:SetMovable(false)
			self.Frame:EnableMouse(false)
			self.Frame:RegisterForDrag()
			self.Frame:SetScript("OnDragStart", nil)
			self.Frame:SetScript("OnDragStop", nil)
			self:Hide()
		end,
		---@param self Bar
		["Unlock"] = function(self)
			self.Frame:SetMovable(true)
			self.Frame:EnableMouse(true)
			self.Frame:RegisterForDrag("LeftButton")
			self.Frame:SetScript("OnDragStart", function(bar)
				bar:StartMoving()
				self.HasMoved = true
			 end)
			self.Frame:SetScript("OnDragStop", self.Frame.StopMovingOrSizing)
			self:Show()
		end,
		---@param self Bar
		["SetBorderSize"] = function(self, BorderThickness)
			local offset = BorderThickness * 2
			local template = SchtenksBars.UI:GetBorderTemplate(BorderThickness)
			self.Border:SetWidth(self.BarFrame:GetWidth() + offset)
			self.Border:SetHeight(self.BarFrame:GetHeight() + offset)
			self.IconBorder:SetWidth(self.Icon:GetWidth() + offset)
			self.IconBorder:SetHeight(self.Icon:GetHeight() + offset)
			self.Border:SetBackdrop(template)
			self.IconBorder:SetBackdrop(template)
		end,
		---@param self Bar
		["SetBorderColor"] = function(self, Color)
			self.Border:SetBackdropBorderColor(Color.R, Color.G, Color.B, Color.A)
			self.IconBorder:SetBackdropBorderColor(Color.R, Color.G, Color.B, Color.A)
		end,
		---@param self Bar
		["SetSize"] = function(self, Width, Height, BorderThickness, IconEnabled)
			self.Frame:SetSize(Width, Height)
			local borderOffset = BorderThickness * 2
			Height = Height - borderOffset
			Width = Width - borderOffset - (IconEnabled and BorderThickness or 0)
			self.BarFrame:SetHeight(Height)
			self.BarFrame:SetWidth(Width - (IconEnabled and Height or 0))
			self.Icon:SetSize(Height, Height)
			self.Spark:SetSize(Height, Height * 2)
		end,
		---@param self Bar
		["UpdateFromSettings"] = function(self, settings, setDefaults)
			SchtenksBars.UI:UpdateFromSettings(self, settings, setDefaults)
		end
	}
	return Bar
end

---@return Bar
function SchtenksBars.UI:BarConstructor(BarName, child)
	---@type Bar
	local self = child
	self = (self or SchtenksBars.UI:CreateBarBase())

	self.BarID = Const.Name..BarName
	---@type Frame
	self.Frame = CreateFrame("Frame", self.BarID, UIParent)
	---@type StatusBar
	self.BarFrame = CreateFrame("StatusBar", self.BarID.."StatusBar", self.Frame)
	self.BarFrame:SetFrameStrata("MEDIUM")
	self.BarFrame:Show()

	---@type BackdropTemplateMixin
	self.Border = CreateFrame("Frame", nil, self.BarFrame, "BackdropTemplate")
	self.Border:SetPoint("CENTER", self.BarFrame, "CENTER", 0, 0)
	self.Border:SetFrameStrata("HIGH")

	---@type Frame
	self.Icon = CreateFrame("Frame", nil, self.Frame)

	---@type Texture
	self.IconTexture = self.Icon:CreateTexture(nil, "BACKGROUND", nil, -1)
	self.IconTexture:SetAllPoints()

	---@type BackdropTemplateMixin
	self.IconBorder = CreateFrame("Frame", nil, self.Icon, "BackdropTemplate")
	self.IconBorder:SetPoint("CENTER", self.Icon, "CENTER", 0, 0)
	self.IconBorder:SetFrameStrata("HIGH")

	---@type Texture
	self.Background = self.BarFrame:CreateTexture(nil, "BACKGROUND", nil)
	self.Background:SetAllPoints()

	---@type Texture
	self.Spark = self.BarFrame:CreateTexture(nil, "LOW", nil, 1)
	self.Spark:SetTexture(Const.UI.SparkTexture)
	self.Spark:SetBlendMode("ADD")

	---@type FontString
	self.NameText = self.BarFrame:CreateFontString()
	---@type FontString
	self.DurationText = self.BarFrame:CreateFontString()
	self.BarName = BarName
	self:Hide()
	return self
end

---@return Bar
function SchtenksBars.UI:CreateBar(BarName)
	local self = SchtenksBars.UI:BarConstructor(BarName)
	self:UpdateFromSettings(nil , true)
	return self
end





