local LSM = LibStub("LibSharedMedia-3.0")

---@type SchtenksBarsConsts
local Const = SchtenksBars.Const

local function GetNonInterruptableSettings(BarName)
	local settings = SchtenksBars.DB.profile[BarName]
	BarName = (settings[Const.DB.UseGeneral] and Const.General or BarName)
	settings = SchtenksBars.DB.profile[BarName]
	if settings[Const.DB.EnableInterruptBorder] then
		return settings[Const.DB.InterruptBorderColor] , settings[Const.DB.InterruptBorderThickness]
	end
	return settings[Const.DB.BorderColor], settings[Const.DB.BorderThickness]
 end

local function CreateCastBar()
	---@class CastBar
	local CastBar = {
		Type = "CastBar", 
		IsChannel = false,
		ReverseChannel = false,
		CanUseHostility = true,
		---@param self CastBar
		["SetDurationText"] = function(self, Duration)
			local min, max = self.BarFrame:GetMinMaxValues()
			if not self.IsChannel then Duration = max - Duration end
			self.DurationText:SetText(string.format("%.1f", Duration))
		end,
		---@param self CastBar
		["SetInfo"] = function(self, name, unit, class, isChannel, notInterruptable)
			local _, fontSize, _ = self.NameText:GetFont()
			local nameLength = (self.BarFrame:GetWidth() - (fontSize)) /(fontSize/1.5)
			local postFix = strlen(name) > nameLength and "..." or ""
			self.NameText:SetText(string.gsub(string.sub(name, 0, nameLength-1),  '[ \t]+%f[\r\n%z]', '')..postFix)
			local FGColor, BGColor = SchtenksBars.UI:GetBarColors(nil, unit, class, self.CanUseHostility)
			self:SetForegroundColor(FGColor)
			self:SetBackgroundColor(BGColor)
			local color, thickness = nil, nil
			if notInterruptable then
				color, thickness = GetNonInterruptableSettings(unit)
			else
				if SchtenksBars.DB.profile[self.BarName][Const.DB.UseGeneral] then
					color = SchtenksBars.DB.profile[Const.General][Const.DB.BorderColor]
					thickness = SchtenksBars.DB.profile[Const.General][Const.DB.BorderThickness]
				else
					color = SchtenksBars.DB.profile[self.BarName][Const.DB.BorderColor]
					thickness = SchtenksBars.DB.profile[self.BarName][Const.DB.BorderThickness]
				end
			end
			self:SetBorderSize(thickness)
			self:SetBorderColor(color)
			self.IsChannel = isChannel or false
		end,
		---@param self CastBar
		["Start"] = function(self, StartTime)
			if self.Enabled then
				self.Frame:Show()
				self.Frame:SetScript("OnUpdate", (function()
					self:SetDuration(GetTime() - StartTime)
				end))
			end
		end,
		---@param self CastBar
		["Stop"] = function(self)
			self.Frame:SetScript("OnUpdate", nil)
			self.Frame:Hide()
		end,
		---@param self CastBar
		["UpdateFromSettings"] = function (self, settings, setDefaults)
			if settings == nil then
				settings = {}
				if SchtenksBars.DB.profile[self.BarName][Const.DB.UseGeneral] then
					settings = SchtenksBars:GetSpecificsForBar(settings, self.BarName)
					setmetatable(settings, {__index = SchtenksBars.DB.profile[Const.General]})
				else
					settings = SchtenksBars.DB.profile[self.BarName]
				end
			end
			self.ReverseChannel = settings[Const.DB.ReverseChannel]
			SchtenksBars.UI:UpdateFromSettings(self, settings, setDefaults)
		end
	}
	---@type Bar 
	local bar = SchtenksBars.UI:CreateBarBase()
	setmetatable(CastBar, {__index = bar})
	return CastBar
end

---@return CastBar
function SchtenksBars.UI:CreateCastBar(BarName)
	local self = SchtenksBars.UI:BarConstructor(BarName, CreateCastBar())
	self:UpdateFromSettings(nil, true)
	return self
end