local LSM = LibStub("LibSharedMedia-3.0")

---@type SchtenksBarsConsts
local Const = SchtenksBars.Const

local function CreateMirrorBars()
	---@class MirrorBars
	local MirrorBars = {
		Type = "MirrorBars",
		Enabled = false,
		IconEnabled = false,
		SparkEnabled = false,
		UseBlizzard = false,
		CanUseHostility = false,
		BarName = Const.Mirror,
		NrBars = MIRRORTIMER_NUMTIMERS,
		---@type MirorBar[]
		Bars = {},
		BarKeys = {},
		---@type boolean[]
		BarsPaused = {},
		---@type Frame
		MirrorFrame = nil,
		GrowDirection = "",
		---@param self MirrorBars
		["SetNameText"] = function(self, Name, index)
			if index == nil then
				for i = 1, self.NrBars, 1 do
					local bar = self.Bars[i]
					bar:SetNameText(Name)
				end
			else
				local bar = self.Bars[index]
				bar:SetNameText(Name)
			end
		end,
		---@param self MirrorBars
		["SetDuration"] = function(self, Duration, index)
			if index == nil then
				for i = 1, self.NrBars, 1 do
					local bar = self.Bars[i]
					bar:SetDuration(Duration)
				end
			else
				local bar = self.Bars[index]
				bar:SetDuration(Duration)
			end
		end,
		---@param self MirrorBars
		["SetFullDuration"] = function(self, Duration, index)
			if index == nil then
				for i = 1, self.NrBars, 1 do
					local bar = self.Bars[i]
					bar:SetFullDuration(Duration)
				end
			else
				local bar = self.Bars[index]
				bar:SetFullDuration(Duration)
			end
		end,
		---@param self MirrorBars
		["SetIcon"] = function(self, Texture, index)
			if index == nil then
				for i = 1, self.NrBars, 1 do
					local bar = self.Bars[i]
					bar:SetIcon(Texture)
				end
			else
				local bar = self.Bars[index]
				bar:SetIcon(Texture)
			end
		end,
		---@param self MirrorBars
		["SetForegroundColor"] = function(self, Color, index)
			if index == nil then
				for i = 1, self.NrBars, 1 do
					local bar = self.Bars[i]
					bar:SetForegroundColor(Color)
				end
			else
				local bar = self.Bars[index]
				bar:SetForegroundColor(Color)
			end
		end,
		---@param self MirrorBars
		["SetBackgroundColor"] = function(self, Color, index)
			if index == nil then
				for i = 1, self.NrBars, 1 do
					local bar = self.Bars[i]
					bar:SetBackgroundColor(Color)
				end
			else
				local bar = self.Bars[index]
				bar:SetBackgroundColor(Color)
			end
		end,
		---@param self MirrorBars
		["SetSubPositions"] = function(self, BorderThickness, IconOnTheRight)
			local direction = 1
			local AnchorPoint = "BOTTOMLEFT"
			if self.GrowDirection == "Down" then
				AnchorPoint = "TOPLEFT"
				direction = -1
			end
			
			for i = 1, self.NrBars, 1 do
				local bar = self.Bars[i]
				bar:SetPosition(AnchorPoint, self.MirrorFrame, AnchorPoint, 0, direction * (i - 1) * bar.Frame:GetHeight())
				bar:SetSubPositions(BorderThickness, IconOnTheRight)
			end
		end,
		---@param self MirrorBars
		["SetClampedToScreen"] = function(self, clamped)
			self.MirrorFrame:SetClampedToScreen(clamped)
		end,
		---@param self MirrorBars
		["SetPosition"]= function(self, AnchorPoint, AnchorFrame, AnchorToPoint, X, Y)
			self.MirrorFrame:ClearAllPoints()
			if (AnchorFrame == "") or (AnchorFrame == nil) then
				AnchorFrame = "UIParent"
			end
			self.MirrorFrame:SetPoint(AnchorPoint, AnchorFrame, AnchorToPoint, X, Y)
			self.MirrorFrame:SetParent(AnchorFrame)
		end,
		---@param self MirrorBars
		["Show"] = function(self)
			self.MirrorFrame:Show()
			self:ShowBar()
		end,
		---@param self MirrorBars
		["Hide"] = function(self)
			self.MirrorFrame:Hide()
			self:HideBar()
		end,
		---@param self MirrorBars
		["ShowBar"] = function(self, index)
			if index == nil then
				for i = 1, self.NrBars, 1 do
					local bar = self.Bars[i]
					bar:Show()
				end
			else
				local bar = self.Bars[index]
				bar:Show()
			end
		end,
		["HideBar"] = function(self, index)
			if index == nil then
				for i = 1, self.NrBars, 1 do
					local bar = self.Bars[i]
					bar:Hide()
				end
			else
				local bar = self.Bars[index]
				bar:Hide()
			end
		end,
		---@param self MirrorBars
		["SetTimer"] = function(self, timer, index)
			self.Bars[index].Timer = timer
		end,
		---@param self MirrorBars
		["SetPaused"] = function(self, paused, index)
			self.Bars[index].Paused = paused
			self.BarsPaused[index] = index
		end,
		["GetLeft"] = function(self)
			return self.MirrorFrame:GetLeft()
		end,
		["GetTop"] = function(self)
			return self.MirrorFrame:GetTop()
		end,
		["GetHeight"] = function(self)
			return self.MirrorFrame:GetHeight()
		end,
		---@param self MirrorBars
		["Lock"] = function(self)
			self.MirrorFrame:SetMovable(false)
			self.MirrorFrame:EnableMouse(false)
			self.MirrorFrame:RegisterForDrag()
			self.MirrorFrame:SetScript("OnDragStart", nil)
			self.MirrorFrame:SetScript("OnDragStop", nil)
			self:HideBar()
		end,
		---@param self MirrorBars
		["Unlock"] = function(self)
			self:ShowBar()
			self.MirrorFrame:SetMovable(true)
			self.MirrorFrame:EnableMouse(true)
			self.MirrorFrame:RegisterForDrag("LeftButton")
			self.MirrorFrame:SetScript("OnDragStart", function(frame)
				frame:StartMoving()
				self.HasMoved = true
			 end)
			self.MirrorFrame:SetScript("OnDragStop", self.MirrorFrame.StopMovingOrSizing)
			
		end,
		---@param self MirrorBars
		["SetBorderSize"] = function(self, BorderThickness)
			for i = 1, self.NrBars, 1 do
				local bar = self.Bars[i]
				bar:SetBorderSize(BorderThickness)
			end
		end,
		---@param self MirrorBars
		["SetSize"] = function(self, Width, Height, BorderThickness)
			local MirrorFrameWidth = Width
			local MirrorFrameHeight = (Height * 3)
			self.MirrorFrame:SetSize(MirrorFrameWidth, MirrorFrameHeight)
			for i = 1, self.NrBars, 1 do
				local bar = self.Bars[i]
				bar:SetSize(Width, Height, BorderThickness, self.IconEnabled)
			end
		end,
		---@param self MirrorBars
		["UpdateFromSettings"] = function(self, settings, setDefaults)
			if settings == nil then
				self.GrowDirection = SchtenksBars.DB.profile[self.BarName][Const.DB.GrowDirection]
				self.UseBlizzard = SchtenksBars.DB.profile[self.BarName][Const.DB.UseBlizzard]
			else
				self.GrowDirection = settings[Const.DB.GrowDirection]
				self.UseBlizzard = settings[Const.DB.UseBlizzard]
			end
			SchtenksBars.UI:UpdateFromSettings(self, settings)
			for i = 1, self.NrBars, 1 do
				if self.BarKeys[i] == nil or setDefaults then
					self:SetIcon(Const.UI.DefaultIcon, i)
					self:SetNameText("Mirror Bar "..i, i)
					self:SetDuration(i * 2, i)
				else
					if self.UseBlizzard then
						local color = MirrorTimerColors[self.BarKeys[i]]
						self:SetForegroundColor({R = color.r, G = color.g, B = color.b, A = 1}, i)
						self:SetBackgroundColor({R = color.r * 0.4, G = color.g * 0.4, B = color.b * 0.4, A = 1}, i)
					end
				end
			end
		end,
		---@param self MirrorBars
		["Start"] = function(self, timer, value, maxvalue, scale, paused, timerLabel)
			if not (self.BarKeys[timer] == nil) then return end
			
			for i = 1, self.NrBars, 1 do
				if self.BarKeys[i] == nil then
					self.BarKeys[i] = timer
					self.BarKeys[timer] = i
					self:SetPaused(paused == 1, i)
					self:SetFullDuration(maxvalue/1000, i)
					self:SetDuration(value, i)
					self:SetNameText(timerLabel, i)
					self:SetTimer(timer, i)
					self:ShowBar(i)
					local icon = 0
					if timer == "BREATH" then
						icon = 136148
					elseif timer == "FEIGNDEATH" then
						icon = 132293
					else
						icon = 136213
					end
					if self.UseBlizzard then
						local color = MirrorTimerColors[timer]
						self:SetForegroundColor({R = color.r, G = color.g, B = color.b, A = 1}, i)
						self:SetBackgroundColor({R = color.r * 0.4, G = color.g * 0.4, B = color.b * 0.4, A = 1}, i)
					end
					self:SetIcon(icon, i)
					self.Bars[i].Frame:SetScript("OnUpdate", (function()
						if self.Bars[i].Paused then return end
						self:SetDuration(GetMirrorTimerProgress(self.Bars[i].Timer)/1000, i)
					end))
					return
				end
			end
		end,
		---@param self MirrorBars
		["Pause"] = function(self, timer, pauses)
			if self.BarKeys[timer] == nil then return end
			self.Stop(timer)
		end,
		---@param self MirrorBars
		["Stop"] = function(self, timer)
			if self.BarKeys[timer] == nil then return end
			local i = self.BarKeys[timer]
			self:SetPaused(false, i)
			self.BarKeys[i] = nil
			self.BarKeys[timer] = nil
			self.Bars[i].Frame:SetScript("OnUpdate", nil)
			self:HideBar(i)
		end
		
	}
	setmetatable(MirrorBars, {
		__index = function(self, key)
			return function(...)
				for i = 1, self.NrBars, 1 do
					local bar = self.Bars[i]
					bar[key](bar, select(2, ...))
				end
			end
		end
	})
	---@type Frame
	MirrorBars.MirrorFrame = CreateFrame("Frame", Const.Name.."MirrorFrame", UIParent)
	MirrorBars.MirrorFrame:SetFrameStrata("BACKGROUND")
	for i = 1, MirrorBars.NrBars, 1 do
		---@class MirorBar
		local bar = SchtenksBars.UI:BarConstructor(Const.Mirror)
		bar.Timer = ""
		bar.Paused = false
		bar.Type = "MirrorBar"

		MirrorBars.Bars[i] = bar
		MirrorBars:HideBar(i)
		MirrorBars.BarKeys[i] = nil
		MirrorBars:SetPaused(false, i)
	end
	return MirrorBars
end

---@return MirrorBars
function SchtenksBars.UI:CreateMirrorBars()
	---@type MirrorBars
	local self = CreateMirrorBars()
	self:UpdateFromSettings(nil, true)
	return self
end