local AceDB = LibStub("AceDB-3.0")

---@class SchtenksBars
SchtenksBars = select(2, ...)

---@class SchtenksBarsConsts
local Const = {}
--#region General Consts
Const.Name =  "SchtenksBars"
Const.DBName = Const.Name.."DB"
Const.General= "general"
Const.Player = "player"
Const.Target = "target"
Const.Focus = "focus"
Const.Profiles = "profiles"
Const.CastBars = "castbars"
Const.Mirror = "mirror"
SchtenksBars.Const = Const

SchtenksBars.Bars = {}
---@type CastBar[]
SchtenksBars.Bars[Const.CastBars] = {}
---@type MirrorBars
SchtenksBars.Bars[Const.Mirror] = {}

---@type Frame
SchtenksBars.Frame = CreateFrame("FRAME", SchtenksBars.Const.Name)
---@type Events
SchtenksBars.Events  = {}

--#region Help Functions
function Propercase (str)
	return str:gsub("^%l", string.upper)
end

---Add Iterator functions to castbar table
SchtenksBars.TableFunctions = {
	Keys = function(self)
		local i = 0
		return function ()
			i = i + 1
			local k = self[i]
			if k ~= nil then return k end
		end
	end,
	Values = function(self)
		local i = 0
		return function ()
			i = i + 1
			local k = self[i]
			if k ~= nil then return self[k] end
		end
	end,
	Pairs = function(self)
		local i = 0
		return function ()
			i = i + 1
			local k = self[i]
			if k ~= nil then return k, self[k] end
		end
	end,
	Add = function(self, k, v)
		if k ~= nil then
			self[k] = v
			self[#self + 1] = k
		end
		return self
	end
}
setmetatable(SchtenksBars.Bars[Const.CastBars], {__index = SchtenksBars.TableFunctions})
--#endregion

function SlashCmdList.SCHTENKSBARS(msg)
	msg = string.lower(msg)
	local actions =
	{
		[Const.Mirror] = function() SchtenksBars:CreateConfigWindow(Const.Mirror) end
	}
	((actions[msg]) or (function() 
		SchtenksBars:CreateConfigWindow()
	end))()
end

SLASH_SCHTENKSBARS1 = "/schtenksbars"
SLASH_SCHTENKSBARS2 = "/scb"

function SchtenksBars.Events:PLAYER_ENTERING_WORLD()

	SchtenksBars.DB = AceDB:New(Const.DBName, SchtenksBars:CreateDBDefaults(), true)
	SchtenksBars.DB:RegisterCallback("OnProfileChanged", function(self, name)
		---@type CastBar
		for bar in SchtenksBars.Bars[Const.CastBars]:Values() do
			bar:UpdateFromSettings()
		end
		SchtenksBars.Bars[Const.Mirror]:UpdateFromSettings()
	end)
	print("merge tradeskill, mirror setting change, fix interrupt")

	SchtenksBars.Bars[Const.CastBars]:Add(Const.Player, SchtenksBars.UI:CreateCastBar(Const.Player))
	SchtenksBars.Bars[Const.CastBars]:Add(Const.Target, SchtenksBars.UI:CreateCastBar(Const.Target))
	SchtenksBars.Bars[Const.CastBars]:Add(Const.Focus, SchtenksBars.UI:CreateCastBar(Const.Focus))
	SchtenksBars.Bars[Const.Mirror] = SchtenksBars.UI:CreateMirrorBars()
end