---@type AceGUI
local AceGUI = LibStub("AceGUI-3.0")
---@type AceDBOptions
local AceDBOptions = LibStub("AceDBOptions-3.0")
---@type AceConfigRegistry
local AceConfigRegistry = LibStub("AceConfigRegistry-3.0", false)
---@type AceConfigDialog
local AceConfigDialog = LibStub("AceConfigDialog-3.0", false)
local LSM = LibStub("LibSharedMedia-3.0")

---@type SchtenksBarsConsts
local Const = SchtenksBars.Const

SchtenksBars.Config = {}
SchtenksBars.Config.Tooltip = CreateFrame("GameTooltip", Const.Name.."ConfigTooltip", UIParent, "GameTooltipTemplate")

--#region Const.Config
Const.Config = {}
Const.Config.ConfigFrame = Const.Name.."Config"
Const.Config.ColumnWidth = 140
Const.Config.BtnWidth = 120
Const.Config.AnchorPoints = {
	["TOPLEFT"] = "Top Left",
	["TOP"] = "Top",
	["TOPRIGHT"] = "Top Right",
	["LEFT"] = "Left",
	["CENTER"] = "Center",
	["RIGHT"] = "Right",
	["BOTTOMLEFT"] = "Bottom Left",
	["BOTTOM"] = "Bottom",
	["BOTTOMRIGHT"] = "Bottom Right"
}
Const.Config.LSM_Widgets = {
	["LSM30_Font"] = "font",
	["LSM30_Sound"] = "sound",
	["LSM30_Statusbar"] = "statusbar",
	["LSM30_Background"] = "background",
	["LSM30_Border"] = "border"
}
--#endregion

--#region Const.DB
Const.DB = {}
Const.DB.Enabled = "Enabled"
Const.DB.UseGeneral = "UseGeneral"
Const.DB.MergeTradeskill = "MergeTradeSkill"
Const.DB.ReverseFill = "ReverseFill"
Const.DB.ReverseChannel = "ReverseChannel"
Const.DB.BarTexture = "BarTexture"
Const.DB.EnableClass = "EnableClass"
Const.DB.ShowSpark = "ShowSpark"
Const.DB.BackgroundColor = "BackgroundColor"
Const.DB.ForegroundColor = "ForegroundColor"
Const.DB.Width = "Width"
Const.DB.Heigth = "Height"
Const.DB.X = "X"
Const.DB.Y = "Y"
Const.DB.AnchorPoint = "AnchorPoint"
Const.DB.AnchorToPoint = "AnchorToPoint"
Const.DB.AnchorFrame = "AnchorFrame"
Const.DB.Font = "Font"
Const.DB.FontSize = "FontSize"
Const.DB.ShowIcon = "ShowIcon"
Const.DB.IconOnTheRight = "IconOnTheRight"
Const.DB.IconZoom = "IconZoom"
Const.DB.BorderColor = "BorderColor"
Const.DB.BorderThickness = "BorderThickness"
Const.DB.EnableInterruptBorder = "EnableInterruptBorder"
Const.DB.InterruptBorderColor = "InterruptBorderColor"
Const.DB.InterruptBorderThickness = "InterruptBorderThickness"
Const.DB.GrowDirection = "GrowDirection"
Const.DB.UseBlizzard = "UseBlizzard"
Const.DB.HostilityBased = "HostilityBased"
--#endregion

function SchtenksBars:CreateDBDefaults()
	local Defaults = {
		profile = {
			['**'] = {
				[Const.DB.Enabled] = true,
				[Const.DB.UseGeneral] = false,
				[Const.DB.ReverseFill] = false,
				[Const.DB.ReverseChannel] = true,
				[Const.DB.BarTexture] = "Aluminium",
				[Const.DB.EnableClass] = true,
				[Const.DB.HostilityBased] = true,
				[Const.DB.ShowSpark] = true,
				[Const.DB.BackgroundColor] = {R = 0, G = 0, B = 0, A = 1},
				[Const.DB.ForegroundColor] = {R = 1, G = 1, B = 1, A = 1},
				[Const.DB.Width] = 150,
				[Const.DB.Heigth] = 25,
				[Const.DB.X] = 400,
				[Const.DB.Y] = 400,
				[Const.DB.AnchorPoint] = "BOTTOMLEFT",
				[Const.DB.AnchorToPoint] = "BOTTOMLEFT",
				[Const.DB.AnchorFrame] = "UIParent",
				[Const.DB.Font] = "Friz Quadrata TT",
				[Const.DB.FontSize] = 12,
				[Const.DB.ShowIcon] = true,
				[Const.DB.IconOnTheRight] = false,
				[Const.DB.IconZoom] = 0.15,
				[Const.DB.BorderColor] = {R = 0, G = 0, B = 0, A = 1},
				[Const.DB.BorderThickness] = 1.8,
				[Const.DB.EnableInterruptBorder] = true,
				[Const.DB.InterruptBorderColor] = {R = 0.7, G = 0.7, B = 0.7, A = 1},
				[Const.DB.InterruptBorderThickness] = 2.5
			},
			[Const.Player] = {
				[Const.DB.MergeTradeskill] = true
			},
			[Const.Mirror] = {
				[Const.DB.GrowDirection] = "Up",
				[Const.DB.UseBlizzard] = false
			},
		}
	}
	return Defaults
end

function SchtenksBars:GetSpecificsForBar(settings, barName)
	settings[Const.DB.Enabled] = SchtenksBars.DB.profile[barName][Const.DB.Enabled]
	settings[Const.DB.UseGeneral] = SchtenksBars.DB.profile[barName][Const.DB.UseGeneral]
	settings[Const.DB.ReverseChannel] = SchtenksBars.DB.profile[barName][Const.DB.ReverseChannel]
	settings[Const.DB.X] = SchtenksBars.DB.profile[barName][Const.DB.X]
	settings[Const.DB.Y] = SchtenksBars.DB.profile[barName][Const.DB.Y]
	settings[Const.DB.AnchorPoint] = SchtenksBars.DB.profile[barName][Const.DB.AnchorPoint]
	settings[Const.DB.AnchorFrame] = SchtenksBars.DB.profile[barName][Const.DB.AnchorFrame]
	settings[Const.DB.AnchorToPoint] = SchtenksBars.DB.profile[barName][Const.DB.AnchorToPoint]
	return settings
end

local function ShowAllBars(show)
	---@type Bar
	for bar in SchtenksBars.Bars[SchtenksBars.CurrentConfigType]:Values() do	
		if show then bar:Show() else bar:Hide() end
	end
end
---@param bar Bar
local function UpdatePositionSettings(bar)
	if bar.HasMoved then
		SchtenksBars.DB.profile[bar.BarName][Const.DB.AnchorFrame] = "UIParent"
		SchtenksBars.DB.profile[bar.BarName][Const.DB.AnchorPoint] = "BOTTOMLEFT"
		SchtenksBars.DB.profile[bar.BarName][Const.DB.AnchorToPoint] = "BOTTOMLEFT"
		SchtenksBars.DB.profile[bar.BarName][Const.DB.X] = bar:GetLeft()
		SchtenksBars.DB.profile[bar.BarName][Const.DB.Y] = bar:GetTop() - bar:GetHeight()
	end
end

local function LockBar(barName, Locked)
	if barName == Const.General then
		---@type Bar
		if not Locked then SchtenksBars.ConfigFrame:SetStatusText(Propercase(SchtenksBars.CurrentConfigType).." Is Unlocked") end
		for bar in SchtenksBars.Bars[SchtenksBars.CurrentConfigType]:Values() do
			if Locked then
				bar:Lock()
				UpdatePositionSettings(bar)
				bar:UpdateFromSettings()
			else 
				bar:Unlock()
				bar:SetBorderColor({R = 1, G = 0, B = 0, A = 1})
				bar:SetForegroundColor({R = 1, G = 1, B = 1, A = 1})
			end
		end
	else
		---@type Bar
		local bar
		if barName == SchtenksBars.CurrentConfigType then
			bar = SchtenksBars.Bars[SchtenksBars.CurrentConfigType]
		else
			bar = SchtenksBars.Bars[SchtenksBars.CurrentConfigType][barName]
		end
		if Locked then
			bar:Lock()
			UpdatePositionSettings(bar)
			bar:UpdateFromSettings()
		else
			bar:Unlock()
			bar:SetBorderColor({R = 1, G = 0, B = 0, A = 1})
			bar:SetForegroundColor({R = 1, G = 1, B = 1, A = 1})
			SchtenksBars.ConfigFrame:SetStatusText(Propercase(SchtenksBars.CurrentConfigType).." Is Unlocked")
		end
	end
end

local function OnValueChanged(name, key, value)
	if name == Const.General then
		---@type Bar
		for barName, bar in SchtenksBars.Bars[SchtenksBars.CurrentConfigType]:Pairs() do
			if SchtenksBars.DB.profile[barName][Const.DB.UseGeneral] then
				local settings = {}
				settings = SchtenksBars:GetSpecificsForBar(settings, barName)
				settings[key] = value
				setmetatable(settings, {__index = SchtenksBars.DB.profile[Const.General]})
				bar:UpdateFromSettings(settings)
			end
		end
	else
		local settings = {}
		settings =  SchtenksBars:GetSpecificsForBar(settings, name)
		settings[key] = value
		if SchtenksBars.DB.profile[name][Const.DB.UseGeneral] then
			setmetatable(settings, {__index =  SchtenksBars.DB.profile[Const.General]})
		else
			setmetatable(settings, {__index = SchtenksBars.DB.profile[name]})
		end
		if name == SchtenksBars.CurrentConfigType then
			bar = SchtenksBars.Bars[SchtenksBars.CurrentConfigType]
		else
			bar = SchtenksBars.Bars[SchtenksBars.CurrentConfigType][name]
		end
		bar:UpdateFromSettings(settings)
	end
end

local function OnValueSet(name, key, value, exclusiveToKey, exclusiveValue)
	SchtenksBars.DB.profile[name][key] = value
	OnValueChanged(name, key, value)
	if exclusiveToKey ~= nil then
		SchtenksBars.DB.profile[name][exclusiveToKey] = exclusiveValue
		OnValueChanged(name, exclusiveToKey, exclusiveValue)
	end
end

---@param widget AceWidgetBase
---@param exclusiveTo AceWidgetCheckBox
local function SetValueAndCreateCallBack(widget, DBKey, name, exclusiveTo, exclusiveToKey)
	if widget.type == "ColorPicker" then
		local color = SchtenksBars.DB.profile[name][DBKey]
		widget:SetColor(color.R, color.G, color.B, color.A)
	elseif widget.type == "EditBox" then
		widget:SetText(SchtenksBars.DB.profile[name][DBKey])
	else
		widget:SetValue(SchtenksBars.DB.profile[name][DBKey])
	end

	if widget.type == "ColorPicker" then
		widget:SetCallback("OnValueChanged", function(self, _, R, G, B, A)
			OnValueChanged(name, DBKey, {R = R, G = G, B = B, A = A})
		end)
		widget:SetCallback("OnValueConfirmed", function(self, _, R, G, B, A)
			OnValueSet(name, DBKey, {R = R, G = G, B = B, A = A})
		end)
	elseif widget.type == "Slider" then
		widget:SetCallback("OnValueChanged", function(self, _, value)
			OnValueChanged(name, DBKey, value)
		end)
		widget:SetCallback("OnMouseUp", function(self, _, value)
			OnValueSet(name, DBKey, value)
		end)
	elseif widget.type == "EditBox" then
		widget:SetCallback("OnEnterPressed", function(self, _, value)
			OnValueSet(name, DBKey, value)
		end)
	else
		---@param self AceWidgetBase
		widget:SetCallback("OnValueChanged", function(self, _, value)
			local exclusiveValue = nil
			if exclusiveToKey ~= nil then
				if exclusiveTo:GetValue() == value and value then
					exlusiveValue = (not value)
				end
			end
			OnValueSet(name, DBKey, value, exclusiveToKey, exclusiveValue)
			if self.type:sub(1, 3) == "LSM" then
				self:SetValue(value)
			end
			if exclusiveToKey ~= nil then
				exclusiveTo:SetValue(exclusiveValue)
			end
		end)
	end
end
---@param widget AceWidgetBase
local function SetHoverDescription(widget, name, description)
	local tooltip = SchtenksBars.Config.Tooltip
	widget:SetCallback("OnEnter", function()
		tooltip:SetOwner(widget.frame, "ANCHOR_TOPRIGHT")
		tooltip:SetText(name, 1, .82, 0, true)
		tooltip:AddLine(description, 1, 1, 1, true)
		tooltip:Show()
	end)
	
	widget:SetCallback("OnLeave", function()
		tooltip:Hide()
	end)

end

---@class Config
---@field Name string
---@field DisplayName string
---@field HasCustomSettings boolean
---@field TabDefinition table
---@field Tabs function[]
---@field CreateConfig function


---@type Config[]
local Configs = {}
setmetatable(Configs, {__index = SchtenksBars.TableFunctions})

Configs:Add(Const.CastBars, {
	Name = Const.CastBars,
	DisplayName = "CastBars",
	HasCustomSettings = true,
	TabDefinition = {
		{--General
			text = "General",
			value = "general"
		},
		{--Appearance
			text = "Appearance",
			value = "appearance"
		},
		{--Size
			text = "Size",
			value = "size"
		},
		{--Position
			text = "Position",
			value = "position"
		},
		{--Font
			text = "Font",
			value = "font"
		},
		{--Icon
			text = "Icon",
			value = "icon"
		},
		{--Border
			text = "Border",
			value = "border"
		},
	},
	Tabs = {
		---@param ScrollFrame AceContainerScrollFrame
		["general"] = function(ScrollFrame)
			local btnWidth = 180
			--#region Buttons
			---@type AceContainerInlineGroup
			local VisibilityGroup = AceGUI:Create("InlineGroup")
			VisibilityGroup:SetTitle("Visibility Controls")
			VisibilityGroup:SetFullWidth("Fill")
			VisibilityGroup:SetLayout("Table")
			VisibilityGroup:SetUserData("table", {
				columns = {btnWidth, btnWidth},
				space = 10,
				align = "LEFT"
			})
			---@type AceWidgetButton
			local ShowAll = AceGUI:Create("Button")
			ShowAll.ShowArg = true
			ShowAll:SetText("Show All Bars")
			ShowAll:SetWidth(btnWidth)
			ShowAll:SetCallback("OnClick", function()
				ShowAllBars(true)
			end)

			---@type AceWidgetButton
			local HideAll = AceGUI:Create("Button")
			HideAll.ShowArg = false
			HideAll:SetText("Hide All Bars")
			HideAll:SetWidth(btnWidth)
			HideAll:SetCallback("OnClick", function()
				ShowAllBars(false)
			end)
			VisibilityGroup:AddChildren(ShowAll, HideAll)

			---@type AceContainerInlineGroup
			local MovableGroup = AceGUI:Create("InlineGroup")
			MovableGroup:SetTitle("Movable Controls")
			MovableGroup:SetFullWidth("Fill")
			MovableGroup:SetLayout("Table")
			MovableGroup:SetUserData("table", {
				columns = {btnWidth, btnWidth},
				space = 10,
				align = "LEFT"
			})

			---@type AceWidgetButton
			local UnlockAll = AceGUI:Create("Button")
			UnlockAll:SetText("Unlock All Bars")
			UnlockAll:SetWidth(btnWidth)
			UnlockAll:SetCallback("OnClick", function()
				LockBar(Const.General, false)
			end)

			---@type AceWidgetButton
			local LockAll = AceGUI:Create("Button")
			LockAll:SetText("Lock All Bars")
			LockAll:SetWidth(btnWidth)
			LockAll:SetCallback("OnClick", function()
				LockBar(Const.General, true)
			end)

			MovableGroup:AddChildren(UnlockAll, LockAll)
			--#endregion
			ScrollFrame:AddChildren(VisibilityGroup, MovableGroup)

			local function AddToggles(name)
				---@type AceContainerInlineGroup
				local ToggleGroup = AceGUI:Create("InlineGroup")
				ToggleGroup:SetTitle(Propercase(name))
				ToggleGroup:SetFullWidth("Fill")
				ToggleGroup:SetLayout("Flow")
	
				---@type AceWidgetCheckBox
				local Enable = AceGUI:Create("CheckBox")
				Enable:SetLabel("Enable")
				SetValueAndCreateCallBack(Enable, Const.DB.Enabled, name)

				ToggleGroup:AddChild(Enable)
				if name == Const.Player then
					Enable:SetWidth(160)
					---@type AceWidgetCheckBox
					local MergeTradeskill = AceGUI:Create("CheckBox")
					MergeTradeskill:SetWidth(200)
					MergeTradeskill:SetLabel("Merge Tradeskill Casts")
					SetValueAndCreateCallBack(MergeTradeskill, Const.DB.MergeTradeskill, name)

					ToggleGroup:AddChild(MergeTradeskill)
				else
					Enable:SetFullWidth("Fill")
				end

				---@type AceWidgetCheckBox
				local UseGeneral = AceGUI:Create("CheckBox")
				UseGeneral:SetFullWidth("Fill")
				UseGeneral:SetLabel("Use General Settings")
				SetValueAndCreateCallBack(UseGeneral, Const.DB.UseGeneral, name)

				---@type AceWidgetCheckBox
				local ReverseFill = AceGUI:Create("CheckBox")
				ReverseFill:SetWidth(160)
				ReverseFill:SetLabel("Reverse Fill Direction")
				SetValueAndCreateCallBack(ReverseFill, Const.DB.ReverseFill, name)

				---@type AceWidgetCheckBox
				local ReverseChannel = AceGUI:Create("CheckBox")
				ReverseChannel:SetWidth(300)
				ReverseChannel:SetLabel("Reverse When Channeling")
				SetValueAndCreateCallBack(ReverseChannel, Const.DB.ReverseChannel, name)
	
				ToggleGroup:AddChildren(UseGeneral, ReverseFill, ReverseChannel)
				ScrollFrame:AddChild(ToggleGroup)
			end
			for k in SchtenksBars.Bars[Const.CastBars]:Keys() do
				AddToggles(k)
			end
		end,
		---@param ScrollFrame AceContainerScrollFrame
		["appearance"] = function(ScrollFrame)
			local function CreateBarGroup(name)
				---@type AceContainerInlineGroup
				local BarGroup = AceGUI:Create("InlineGroup")
				BarGroup:SetTitle(Propercase(name))
				BarGroup:SetFullWidth("Fill")
				BarGroup:SetLayout("Table")
				BarGroup:SetUserData("table", {
					columns = {200, 120, 200},
					space = 15,
					spaceH = 5,
					align = "LEFT"
				})

				---@type AceWidgetLSMStatusBar
				local BarTexture = AceGUI:Create("LSM30_Statusbar")
				BarTexture:SetLabel("Bar Texture")
				BarTexture:SetList()
				SetValueAndCreateCallBack(BarTexture, Const.DB.BarTexture, name)

				---@type AceWidgetCheckBox
				local EnableClass = AceGUI:Create("CheckBox")
				EnableClass:SetWidth(120)
				EnableClass:SetLabel("Use Class Color")
				SetHoverDescription(EnableClass, "Sets color based on casting player units class.")
				SetValueAndCreateCallBack(EnableClass, Const.DB.EnableClass, name)

				---@type AceWidgetCheckBox
				local ShowSpark = AceGUI:Create("CheckBox")
				ShowSpark:SetWidth(120)
				ShowSpark:SetLabel("Show Spark")
				SetValueAndCreateCallBack(ShowSpark, Const.DB.ShowSpark, name)

				---@type AceWidgetColorPicker
				local BackgroundColor = AceGUI:Create("ColorPicker")
				BackgroundColor:SetWidth(120)
				BackgroundColor:SetLabel("Background Color")
				BackgroundColor:SetHasAlpha(true)
				SetValueAndCreateCallBack(BackgroundColor, Const.DB.BackgroundColor, name)

				---@type AceWidgetColorPicker
				local ForegroundColor = AceGUI:Create("ColorPicker")
				ForegroundColor:SetWidth(120)
				ForegroundColor:SetLabel("Foreground Color")
				ForegroundColor:SetHasAlpha(true)
				SetValueAndCreateCallBack(ForegroundColor, Const.DB.ForegroundColor, name)

				if name == Const.Player then
					BarGroup:AddChildren(BarTexture, EnableClass, ShowSpark, BackgroundColor, ForegroundColor)
				else
					local HostilityBased = AceGUI:Create("CheckBox")
					HostilityBased:SetWidth(250)
					HostilityBased:SetLabel("Use Hostility Based Color")
					SetHoverDescription(HostilityBased, "Sets colors based on being hostile/natural/friendly, if checked is used for non-player units when 'Use Class Color' is set.")
					SetValueAndCreateCallBack(HostilityBased, Const.DB.HostilityBased, name)
					BarGroup:AddChildren(BarTexture, EnableClass, ShowSpark, BackgroundColor, ForegroundColor, HostilityBased)
				end
				return BarGroup
			end
			
			ScrollFrame:AddChild(CreateBarGroup("general"))
			for k in SchtenksBars.Bars[Const.CastBars]:Keys() do
				ScrollFrame:AddChild(CreateBarGroup(k))
			end
		end,
		---@param ScrollFrame AceContainerScrollFrame
		["size"] = function(ScrollFrame)
			local function CreateBarGroup(name)
				---@type AceContainerInlineGroup
				local BarGroup = AceGUI:Create("InlineGroup")
				BarGroup:SetTitle(Propercase(name))
				BarGroup:SetFullWidth("Fill")
				BarGroup:SetLayout("Table")
				BarGroup:SetUserData("table", {
						columns = {0.5, 0.5},
						spaceH = 10,
						align = "LEFT"
				})

				---@type AceWidgetSlider
				local BarWidth = AceGUI:Create("Slider")
				BarWidth:SetFullWidth("Fill")
				BarWidth:SetLabel("Width")
				BarWidth:SetSliderValues(10, 2560, 1)
				SetValueAndCreateCallBack(BarWidth, Const.DB.Width, name)

				---@type AceWidgetSlider
				local BarHeight = AceGUI:Create("Slider")
				BarHeight:SetFullWidth("Fill")
				BarHeight:SetLabel("Height")
				BarHeight:SetSliderValues(2, 1440, 1)
				SetValueAndCreateCallBack(BarHeight, Const.DB.Heigth, name)

				BarGroup:AddChildren(BarWidth, BarHeight)
				return BarGroup
			end

			ScrollFrame:AddChild(CreateBarGroup("general"))
			for k in SchtenksBars.Bars[Const.CastBars]:Keys() do
				ScrollFrame:AddChild(CreateBarGroup(k))
			end
		end,
		---@param ScrollFrame AceContainerScrollFrame
		["position"] = function(ScrollFrame)
			for name in SchtenksBars.Bars[Const.CastBars]:Keys() do
				---@type AceContainerInlineGroup
				local BarGroup = AceGUI:Create("InlineGroup")
				BarGroup:SetTitle(Propercase(name))
				BarGroup:SetFullWidth("Fill")
				BarGroup:SetLayout("Table")
				BarGroup:SetUserData("table", {
						columns = {0.5, 0.5},
						space = 10,
						spaceH = 10,
						align = "LEFT"
				})

				---@type AceWidgetSlider
				local BarX = AceGUI:Create("Slider")
				BarX:SetFullWidth("Fill")
				BarX:SetLabel("X")
				BarX:SetSliderValues(-2560,2560,1)
				SetValueAndCreateCallBack(BarX, Const.DB.X, name)
				---@type AceWidgetSlider
				local BarY = AceGUI:Create("Slider")
				BarY:SetFullWidth("Fill")
				BarY:SetLabel("Y")
				BarY:SetSliderValues(-1440,1440,1)
				SetValueAndCreateCallBack(BarY, Const.DB.Y, name)

				---@type AceWidgetDropdown
				local AnchorPoint = AceGUI:Create("Dropdown")
				AnchorPoint:SetList(Const.Config.AnchorPoints)
				AnchorPoint:SetFullWidth("Fill")
				AnchorPoint:SetLabel("Anchor Point")
				SetValueAndCreateCallBack(AnchorPoint, Const.DB.AnchorPoint, name)

				---@type AceWidgetDropdown
				local AnchorToPoint = AceGUI:Create("Dropdown")
				AnchorToPoint:SetList(Const.Config.AnchorPoints)
				AnchorToPoint:SetFullWidth("Fill")
				AnchorToPoint:SetLabel("Anchor To Point")
				SetValueAndCreateCallBack(AnchorToPoint, Const.DB.AnchorToPoint, name)

				---@type AceWidgetEditBox
				local AnchorFrame = AceGUI:Create("EditBox")
				AnchorFrame:SetFullWidth("Fill")
				AnchorFrame:SetLabel("Amchor Frame")
				SetValueAndCreateCallBack(AnchorFrame, Const.DB.AnchorFrame, name)
				
				BarGroup:AddChildren(BarX, BarY, AnchorPoint, AnchorToPoint, AnchorFrame)
				ScrollFrame:AddChild(BarGroup)
			end
		end,
		---@param ScrollFrame AceContainerScrollFrame
		["font"] = function(ScrollFrame)
			local function CreateBarGroup(name)
				---@type AceContainerInlineGroup
				local BarGroup = AceGUI:Create("InlineGroup")
				BarGroup:SetTitle(Propercase(name))
				BarGroup:SetFullWidth("Fill")
				BarGroup:SetLayout("Table")
				BarGroup:SetUserData("table", {
						columns = {0.5, 0.5},
						space = 10,
						align = "LEFT"
				})

				---@type AceWidgetLSMFont
				local BarFont = AceGUI:Create("LSM30_Font")
				BarFont:SetFullWidth("Fill")
				BarFont:SetLabel("Font")
				BarFont:SetList()
				SetValueAndCreateCallBack(BarFont, Const.DB.Font , name)

				---@type AceWidgetSlider
				local FontSize = AceGUI:Create("Slider")
				FontSize:SetFullWidth("Fill")
				FontSize:SetLabel("Font Size")
				FontSize:SetSliderValues(1, 100, 1)
				SetValueAndCreateCallBack(FontSize, Const.DB.FontSize, name)

				BarGroup:AddChildren(BarFont, FontSize)
				return BarGroup
			end
			
			ScrollFrame:AddChild(CreateBarGroup("general"))
			for k in SchtenksBars.Bars[Const.CastBars]:Keys() do
				ScrollFrame:AddChild(CreateBarGroup(Propercase(k)))
			end
		end,
		---@param ScrollFrame AceContainerScrollFrame
		["icon"] = function(ScrollFrame)
			local function CreateBarGroup(name)
				---@type AceContainerInlineGroup
				local BarGroup = AceGUI:Create("InlineGroup")
				BarGroup:SetTitle(Propercase(name))
				BarGroup:SetFullWidth("Fill")
				BarGroup:SetLayout("Flow")

				---@type AceWidgetCheckBox
				local ShowIcon = AceGUI:Create("CheckBox")
				ShowIcon:SetWidth(100)
				ShowIcon:SetLabel("Show Icon")
				SetValueAndCreateCallBack(ShowIcon, Const.DB.ShowIcon, name)

				---@type AceWidgetCheckBox
				local IconRight = AceGUI:Create("CheckBox")
				IconRight:SetWidth(300)
				IconRight:SetLabel("Icon On The Right")
				SetValueAndCreateCallBack(IconRight, Const.DB.IconOnTheRight, name)

				---@type AceWidgetSlider
				local IconZoom = AceGUI:Create("Slider")
				IconZoom:SetWidth(160)
				IconZoom:SetLabel("Icon Zoom")
				IconZoom:SetIsPercent(true)
				IconZoom:SetSliderValues(0.01, 1, 0.01)
				SetValueAndCreateCallBack(IconZoom, Const.DB.IconZoom, name)

				BarGroup:AddChildren(ShowIcon, IconRight, IconZoom)
				return BarGroup
			end
			
			ScrollFrame:AddChild(CreateBarGroup("general"))
			for k in SchtenksBars.Bars[Const.CastBars]:Keys() do
				ScrollFrame:AddChild(CreateBarGroup(k))
			end
		end,
		---@param ScrollFrame AceContainerScrollFrame
		["border"] = function(ScrollFrame)
			local function CreateBarGroup(name)
				---@type AceContainerInlineGroup
				local BarGroup = AceGUI:Create("InlineGroup")
				BarGroup:SetTitle(Propercase(name))
				BarGroup:SetFullWidth("Fill")
				BarGroup:SetLayout("Table")
				BarGroup:SetUserData("table", {
						columns = {180, 200},
						space = 10,
						align = "LEFT"
				})

				---@type AceWidgetColorPicker
				local BorderColor = AceGUI:Create("ColorPicker")
				BorderColor:SetHeight(40)
				BorderColor:SetWidth(140)
				BorderColor:SetLabel("Border Color")
				SetValueAndCreateCallBack(BorderColor, Const.DB.BorderColor, name)

				---@type AceWidgetSlider
				local BorderThickness = AceGUI:Create("Slider")
				BorderThickness:SetWidth(200)
				BorderThickness:SetLabel("Border Thickness")
				BorderThickness:SetSliderValues(0, 10, 0.1)
				SetValueAndCreateCallBack(BorderThickness, Const.DB.BorderThickness, name)

				---@type AceWidgetCheckBox
				local EnableInterruptBorder = AceGUI:Create("CheckBox")
				EnableInterruptBorder:SetWidth(180)
				EnableInterruptBorder:SetLabel("Enable Interrupt Border")
				EnableInterruptBorder:SetUserData("cell", {
					colspan = 2
				})
				SetValueAndCreateCallBack(EnableInterruptBorder, Const.DB.EnableInterruptBorder, name)

				---@type AceWidgetColorPicker
				local InterruptBorderColor = AceGUI:Create("ColorPicker")
				InterruptBorderColor:SetWidth(140)
				InterruptBorderColor:SetHeight(40)
				InterruptBorderColor:SetLabel("Interrupt Border Color")
				SetValueAndCreateCallBack(InterruptBorderColor, Const.DB.InterruptBorderColor, name)

				---@type AceWidgetSlider
				local InterruptBorderThickness = AceGUI:Create("Slider")
				InterruptBorderThickness:SetWidth(200)
				InterruptBorderThickness:SetLabel("Interrupt Border Thickness")
				InterruptBorderThickness:SetSliderValues(0, 10, 0.1)
				SetValueAndCreateCallBack(InterruptBorderThickness, Const.DB.InterruptBorderThickness, name)

				BarGroup:AddChildren(BorderColor, BorderThickness, EnableInterruptBorder, InterruptBorderColor, InterruptBorderThickness)
				return BarGroup
			end
			
			ScrollFrame:AddChild(CreateBarGroup("general"))
			for k in SchtenksBars.Bars[Const.CastBars]:Keys() do
				ScrollFrame:AddChild(CreateBarGroup(k))
			end
		end,
	},
})
Configs:Add(Const.Mirror, {
	Name = Const.Mirror,
	DisplayName = "Mirror",
	HasCustomSettings = true,
	---@param container AceContainerTreeGroup
	CreateConfig = function(self, container)
		local btnWidth = 180
		local name = Const.Mirror 
		---@type AceContainerInlineGroup
		local Group = AceGUI:Create("InlineGroup")
		Group:SetTitle("General")
		Group:SetFullWidth("Fill")
		Group:SetFullHeight("Fill")
		Group:SetLayout("Fill")

		---@type AceContainerScrollFrame
		local ScrollFrame = AceGUI:Create("ScrollFrame")
		ScrollFrame:SetFullHeight("Fill")
		ScrollFrame:SetFullWidth("Fill")

		--#region General
		--#region Buttons
		---@type AceContainerInlineGroup
		local VisibilityGroup = AceGUI:Create("InlineGroup")
		VisibilityGroup:SetTitle("Visibility Controls")
		VisibilityGroup:SetFullWidth("Fill")
		VisibilityGroup:SetLayout("Table")
		VisibilityGroup:SetUserData("table", {
			columns = {btnWidth, btnWidth},
			space = 10,
			align = "LEFT"
		})
		---@type AceWidgetButton
		local ShowMirror = AceGUI:Create("Button")
		ShowMirror:SetText("Show Mirror")
		ShowMirror:SetWidth(btnWidth)
		ShowMirror:SetCallback("OnClick", function()
			SchtenksBars.Bars[Const.Mirror]:ShowBar()
		end)

		---@type AceWidgetButton
		local HideMirror = AceGUI:Create("Button")
		HideMirror:SetText("Hide Mirror")
		HideMirror:SetWidth(btnWidth)
		HideMirror:SetCallback("OnClick", function()
			SchtenksBars.Bars[Const.Mirror]:HideBar()
		end)

		VisibilityGroup:AddChildren(ShowMirror, HideMirror)

		---@type AceContainerInlineGroup
		local MovableGroup = AceGUI:Create("InlineGroup")
		MovableGroup:SetTitle("Movable Controls")
		MovableGroup:SetFullWidth("Fill")
		MovableGroup:SetLayout("Table")
		MovableGroup:SetUserData("table", {
			columns = {btnWidth, btnWidth},
			space = 10,
			align = "LEFT"
		})

		---@type AceWidgetButton
		local UnlockMirror = AceGUI:Create("Button")
		UnlockMirror:SetText("Unlock Mirror")
		UnlockMirror:SetWidth(btnWidth)
		UnlockMirror:SetCallback("OnClick", function()
			LockBar(Const.Mirror, false)
		end)

		---@type AceWidgetButton
		local LockMirror = AceGUI:Create("Button")
		LockMirror:SetText("Lock Mirror")
		LockMirror:SetWidth(btnWidth)
		LockMirror:SetCallback("OnClick", function()
			LockBar(Const.Mirror, true)
		end)

		MovableGroup:AddChildren(UnlockMirror, LockMirror)
		--#endregion

		---@type AceContainerInlineGroup
		local GeneralGroup = AceGUI:Create("InlineGroup")
		GeneralGroup:SetFullWidth("Fill")
		GeneralGroup:SetLayout("Flow")

		---@type AceWidgetCheckBox
		local EnableMirror = AceGUI:Create("CheckBox")
		EnableMirror:SetWidth(100)
		EnableMirror:SetLabel("Enable")
		SetValueAndCreateCallBack(EnableMirror, Const.DB.Enabled, name)
		
		---@type AceWidgetDropdown
		local GrowDirection = AceGUI:Create("Dropdown")
		GrowDirection:SetWidth(200)
		GrowDirection:SetLabel("Grow Direction")
		GrowDirection:SetList({["Up"] = "Up", ["Down"] = "Down"})
		SetValueAndCreateCallBack(GrowDirection, Const.DB.GrowDirection, name)

		GeneralGroup:AddChildren(EnableMirror, GrowDirection)
		--#endregion
		--#region Appearance
		---@type AceContainerInlineGroup
		local AppearanceGroup = AceGUI:Create("InlineGroup")
		AppearanceGroup:SetFullWidth("Fill")
		AppearanceGroup:SetLayout("Table")
		AppearanceGroup:SetUserData("table", {
				columns = {200, 120, 200},
				space = 15,
				spaceH = 10,
				align = "LEFT"
		})

		---@type AceWidgetLSMStatusBar
		local BarTexture = AceGUI:Create("LSM30_Statusbar")
		BarTexture:SetLabel("Bar Texture")
		BarTexture:SetList()
		SetValueAndCreateCallBack(BarTexture, Const.DB.BarTexture, name)


		local UseBlizzard = AceGUI:Create("CheckBox")
		UseBlizzard:SetWidth(200)
		UseBlizzard:SetLabel("Use Blizzard Colors")
		---@type AceWidgetCheckBox
		local EnableClass = AceGUI:Create("CheckBox")
		EnableClass:SetWidth(120)
		EnableClass:SetLabel("Use Class Color")
		SetValueAndCreateCallBack(UseBlizzard, Const.DB.UseBlizzard, name, EnableClass, Const.DB.EnableClass)
		SetValueAndCreateCallBack(EnableClass, Const.DB.EnableClass, name, UseBlizzard, Const.DB.UseBlizzard)

		---@type AceWidgetCheckBox
		local ShowSpark = AceGUI:Create("CheckBox")
		ShowSpark:SetWidth(120)
		ShowSpark:SetLabel("Show Spark")
		SetValueAndCreateCallBack(ShowSpark, Const.DB.ShowSpark, name)

		---@type AceWidgetColorPicker
		local BackgroundColor = AceGUI:Create("ColorPicker")
		BackgroundColor:SetWidth(120)
		BackgroundColor:SetLabel("Background Color")
		BackgroundColor:SetHasAlpha(true)
		SetValueAndCreateCallBack(BackgroundColor, Const.DB.BackgroundColor, name)

		---@type AceWidgetColorPicker
		local ForegroundColor = AceGUI:Create("ColorPicker")
		ForegroundColor:SetWidth(120)
		ForegroundColor:SetLabel("Foreground Color")
		ForegroundColor:SetHasAlpha(true)
		SetValueAndCreateCallBack(ForegroundColor, Const.DB.ForegroundColor, name)

		

		AppearanceGroup:AddChildren(BarTexture, EnableClass, ShowSpark, BackgroundColor, ForegroundColor, UseBlizzard)
		--#endregion
		--#region Size
		---@type AceContainerInlineGroup
		local SizeGroup = AceGUI:Create("InlineGroup")
		SizeGroup:SetFullWidth("Fill")
		SizeGroup:SetLayout("Table")
		SizeGroup:SetUserData("table", {
				columns = {0.5, 0.5},
				spaceH = 10,
				align = "LEFT"
		})

		---@type AceWidgetSlider
		local BarWidth = AceGUI:Create("Slider")
		SetValueAndCreateCallBack(BarWidth, Const.DB.Width, name)
		BarWidth:SetFullWidth("Fill")
		BarWidth:SetLabel("Width")
		BarWidth:SetSliderValues(10, 2560, 1)

		---@type AceWidgetSlider
		local BarHeight = AceGUI:Create("Slider")
		SetValueAndCreateCallBack(BarHeight, Const.DB.Heigth, name)
		BarHeight:SetFullWidth("Fill")
		BarHeight:SetLabel("Height")
		BarHeight:SetSliderValues(2, 1440, 1)

		SizeGroup:AddChildren(BarWidth, BarHeight)
		--#endregion
		--#region Position
		---@type AceContainerInlineGroup
		local PositionGroup = AceGUI:Create("InlineGroup")
		PositionGroup:SetFullWidth("Fill")
		PositionGroup:SetLayout("Table")
		PositionGroup:SetUserData("table", {
				columns = {0.5, 0.5},
				space = 10,
				spaceH = 10,
				align = "LEFT"
		})

		---@type AceWidgetSlider
		local BarX= AceGUI:Create("Slider")
		BarX:SetFullWidth("Fill")
		BarX:SetLabel("X")
		BarX:SetSliderValues(-2560, 2560, 1)
		SetValueAndCreateCallBack(BarX, Const.DB.X, name)

		---@type AceWidgetSlider
		local BarY = AceGUI:Create("Slider")
		BarY:SetFullWidth("Fill")
		BarY:SetLabel("Y")
		BarY:SetSliderValues(-1440, 1440, 1)
		SetValueAndCreateCallBack(BarY, Const.DB.Y, name)

		---@type AceWidgetDropdown
		local AnchorPoint = AceGUI:Create("Dropdown")
		AnchorPoint:SetList(Const.Config.AnchorPoints)
		AnchorPoint:SetFullWidth("Fill")
		AnchorPoint:SetLabel("Anchor Point")
		SetValueAndCreateCallBack(AnchorPoint, Const.DB.AnchorPoint, name)

		---@type AceWidgetDropdown
		local AnchorToPoint = AceGUI:Create("Dropdown")
		AnchorToPoint:SetList(Const.Config.AnchorPoints)
		AnchorToPoint:SetFullWidth("Fill")
		AnchorToPoint:SetLabel("Anchor To Point")
		SetValueAndCreateCallBack(AnchorToPoint, Const.DB.AnchorToPoint, name)

		---@type AceWidgetEditBox
		local AnchorFrame = AceGUI:Create("EditBox")
		AnchorFrame:SetFullWidth("Fill")
		AnchorFrame:SetLabel("Amchor Frame")
		SetValueAndCreateCallBack(AnchorFrame, Const.DB.AnchorFrame, name)

		PositionGroup:AddChildren(BarX, BarY, AnchorPoint, AnchorToPoint, AnchorFrame)
		--#endregion
		--#region Font
		---@type AceContainerInlineGroup
		local FontGroup = AceGUI:Create("InlineGroup")
		FontGroup:SetFullWidth("Fill")
		FontGroup:SetLayout("Table")
		FontGroup:SetUserData("table", {
				columns = {0.5, 0.5},
				space = 10,
				align = "LEFT"
		})

		---@type AceWidgetLSMFont
		local BarFont = AceGUI:Create("LSM30_Font")
		BarFont:SetFullWidth("Fill")
		BarFont:SetLabel("Font")
		BarFont:SetList()
		SetValueAndCreateCallBack(BarFont, Const.DB.Font, name)

		---@type AceWidgetSlider
		local FontSize = AceGUI:Create("Slider")
		FontSize:SetFullWidth("Fill")
		FontSize:SetLabel("Font Size")
		FontSize:SetSliderValues(1, 100, 1)
		FontGroup:AddChildren(BarFont, FontSize)
		SetValueAndCreateCallBack(FontSize, Const.DB.FontSize, name)
		
		
	--#endregion
		--#region Icon
		---@type AceContainerInlineGroup
		local IconGroup = AceGUI:Create("InlineGroup")
		IconGroup:SetFullWidth("Fill")
		IconGroup:SetLayout("Flow")

		---@type AceWidgetCheckBox
		local ShowIcon = AceGUI:Create("CheckBox")
		ShowIcon:SetWidth(100)
		ShowIcon:SetLabel("Show Icon")
		SetValueAndCreateCallBack(ShowIcon, Const.DB.ShowIcon, name)
		
		---@type AceWidgetCheckBox
		local IconRight = AceGUI:Create("CheckBox")
		IconRight:SetWidth(300)
		IconRight:SetLabel("Icon On The Right")
		SetValueAndCreateCallBack(IconRight, Const.DB.IconOnTheRight, name)
		
		---@type AceWidgetSlider
		local IconZoom = AceGUI:Create("Slider")
		IconZoom:SetWidth(160)
		IconZoom:SetLabel("Icon Zoom")
		IconZoom:SetIsPercent(true)
		IconZoom:SetSliderValues(0.01, 1, 0.01)
		IconGroup:AddChildren(ShowIcon, IconRight, IconZoom)
		SetValueAndCreateCallBack(IconZoom, Const.DB.IconZoom, name)
		--#endregion
		--#region Border
		---@type AceContainerInlineGroup
		local BorderGroup = AceGUI:Create("InlineGroup")
		BorderGroup:SetFullWidth("Fill")
		BorderGroup:SetLayout("Table")
		BorderGroup:SetUserData("table", {
				columns = {180, 200},
				space = 10,
				align = "LEFT"
		})

		---@type AceWidgetColorPicker
		local BorderColor = AceGUI:Create("ColorPicker")
		BorderColor:SetHeight(40)
		BorderColor:SetWidth(140)
		BorderColor:SetLabel("Border Color")
		SetValueAndCreateCallBack(BorderColor, Const.DB.BorderColor, name)

		---@type AceWidgetSlider
		local BorderThickness = AceGUI:Create("Slider")
		BorderThickness:SetWidth(200)
		BorderThickness:SetLabel("Border Thickness")
		BorderThickness:SetSliderValues(0, 100, 0.1)
		SetValueAndCreateCallBack(BorderThickness, Const.DB.BorderThickness, name)

		BorderGroup:AddChildren(BorderColor, BorderThickness)
		--#endregion

		ScrollFrame:AddChildren(VisibilityGroup, MovableGroup, GeneralGroup, AppearanceGroup, SizeGroup, PositionGroup, FontGroup, IconGroup, BorderGroup)
		Group:AddChild(ScrollFrame)
		container:AddChild(Group)
	end
})
Configs:Add(Const.Profiles, {
	Name = Const.Profiles,
	DisplayName = "Profiles",
	HasCustomSettings = false,
	---@type AceContainerTreeGroup
	CreateConfig = function(self, container)
		local name = Const.Name.."Profiles"
		if not AceConfigRegistry:GetOptionsTable(name) then
			AceConfigRegistry:RegisterOptionsTable(name, AceDBOptions:GetOptionsTable(SchtenksBars.DB))
		end
		AceConfigDialog:Open(name, container)
	end
})

---@param container AceContainerTreeGroup
local function CreateTabGroup(container, tabs, configName)
	---@type AceContainerTabGroup
	local TabGroup = AceGUI:Create("TabGroup")
	TabGroup:SetLayout("Flow")
	TabGroup:SetTabs(tabs)
	---@param container AceContainerTabGroup
	TabGroup:SetCallback("OnGroupSelected", function(container, _, selected)
		container:ReleaseChildren()
		---@type AceContainerScrollFrame
		local ScrollFrame = AceGUI:Create("ScrollFrame")
		ScrollFrame:SetFullHeight("Fill")
		ScrollFrame:SetFullWidth("Fill")
		Configs[configName].Tabs[selected](ScrollFrame)
		container:AddChild(ScrollFrame)
	end)

	TabGroup:SetFullHeight("Fill")
	TabGroup:SetFullWidth("Fill")
	TabGroup:SelectTab(tabs[1].value)
	container:AddChild(TabGroup)
end

local function CreateNavigationTree(name)
	if name == nil then name = Configs[1] end
	---@type AceContainerTreeGroup
	local NavigationTree = AceGUI:Create("TreeGroup")

	local TreeTable = {}
	local i = 1
	---@type Config
	for key, value in Configs:Pairs() do
		TreeTable[i] = {
			value = key,
			text = value.DisplayName
		}
		i = i + 1
	end
	
	NavigationTree:SetFullHeight(true)
	NavigationTree:SetTree(TreeTable)
	NavigationTree:SetLayout("Fill")
	---@type AceContainerTreeGroup
	NavigationTree:SetCallback("OnGroupSelected", function(container, _, selected)
		SchtenksBars.CurrentConfigType = selected
		local TabDef = Configs[selected].TabDefinition
		container:ReleaseChildren()
		if TabDef then
			CreateTabGroup(container, TabDef, selected)
		else
			Configs[selected]:CreateConfig(container)
		end
		
    end)
	NavigationTree:SelectByPath(name)
	return NavigationTree
end

function SchtenksBars:CreateConfigWindow(name)

	---@type AceContainerFrame
	local Frame = AceGUI:Create("Frame")
	Frame:SetTitle("SchtenksBars")
	Frame:SetHeight(600)
	Frame:SetWidth(800)
	Frame:SetCallback("OnClose", function(widget) AceGUI:Release(widget) end)
	Frame:SetLayout("Flow")
	Frame:EnableResize(false)
	Frame:AddChild(CreateNavigationTree(name))

	-- Register a global name for frame so it can be set as a "special frame"
	-- special frames are closed when the escape key is pressed.
	---@type AceContainerFrame
	SchtenksBars.ConfigFrame = Frame.frame
	_G[Const.Config.ConfigFrame] = SchtenksBars.ConfigFrame
	tinsert(UISpecialFrames, Const.Config.ConfigFrame)
end
